from abc import ABC, abstractmethod
from symbiotic.services.responses import ServiceResponse


class BaseService(ABC):

    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)

    @property
    def name(self) -> str:
        return self.__class__.__name__

    @abstractmethod
    def trigger(self, *args, **kwargs) -> ServiceResponse:
        pass



import logging
from typing import Any

import requests
from schema import And, Optional, Or, Schema
from symbiotic.core.exceptions import ConfigurationError
from symbiotic.devices.parameters import Parameters

from .base import BaseService
from .responses import ServiceResponse


class IFTTT(BaseService):

    DEFAULT_URL = 'https://maker.ifttt.com/trigger/{event_name}/with/key/{key}'

    def __init__(self, config: dict, *args, **kwargs):
        super().__init__(*args, **kwargs)
        try:
            self._url = config.get('url', IFTTT.DEFAULT_URL)
            self._key = config.pop('key')
        except (AttributeError, KeyError):
            raise ConfigurationError(
                'Could not instantiate IFTTT: configuration not found!')

    def _validate_parameters(self, parameters: Any = None) -> dict:
        if parameters is None:
            return {}

        if issubclass(type(parameters), Parameters):
            parameters = parameters.ifttt()

        schema = Schema({
            Optional('value1'): And(Or(int, float, str)),
            Optional('value2'): And(Or(int, float, str)),
            Optional('value3'): And(Or(int, float, str)),
        })

        return schema.validate(parameters)

    def trigger(self, event_name: str, parameters: Any = None) -> ServiceResponse:
        """Triggers the IFTTT webhook 'event_name' with 'parameters'.

        Parameters
        ----------
        event_name : str
            The name of the webhook to trigger, as defined in the "Event Name".

        parameters : dict, optional
            An optional dictionary containing the parameters to pass with the request.
            It can contain one to three parameters defined as follows:
            {
                (Optional) 'value1': '...',
                (Optional) 'value2': '...',
                (Optional) 'value3': '...'
            }

        """
        parameters = self._validate_parameters(parameters)
        url = self._url.format(event_name=event_name, key=self._key)
        response = requests.post(url, parameters)

        logging.debug(f'Request body: {response.request.body}')
        logging.info(f'{response.text}')

        return ServiceResponse.from_response(response)



import logging
import math
from abc import ABC, abstractmethod
from datetime import datetime
from enum import Enum

from symbiotic.services.base import BaseService
from symbiotic.services.responses import ServiceResponse

from .actions import Actionable
from .parameters import LightBulbParameters, Parameters


class SmartDevice(Actionable, ABC):
    """
    SmartDevice encapsulates the methods to control any smart device.

    Devices are controlled through a BaseService; using the facade pattern
    here allows to add more services in the future without refactoring,
    improves readability, and reduces code coupling.
    """

    UPDATES_THROTTLE: int = 5

    class State(Enum):
        ON = 'on'
        OFF = 'off'

        def __str__(self):
            return self.value

    def __init__(self, name: str, *args, **kwargs) -> None:
        self.name: str = name
        self._state: SmartDevice.State = kwargs.pop('state', None)
        self._service: BaseService = kwargs.pop('service', None)
        self._parameters: Parameters = self.default_parameters
        self._last_update = None
        super().__init__(*args, **kwargs)

    "Map device physical states to IFTTT service_event names."
    states_events_mapping: dict = {
        State.ON: 'bedroom_light_color',
        State.OFF: 'switch_off'
    }

    @staticmethod
    def _state_to_service_event(device_state: State):
        return SmartDevice.states_events_mapping[device_state]

    @property
    @abstractmethod
    def default_parameters(self) -> Parameters:
        raise NotImplementedError(
            "The subclass must override 'parameters'.")

    @property
    def parameters(self) -> Parameters:
        return self._parameters

    @property
    def state(self) -> State:
        return self._state

    @state.setter
    def state(self, state: 'SmartDevice.State') -> None:
        self._state = state

    def _change_state(self, state: 'SmartDevice.State', **kwargs) -> ServiceResponse:
        logging.debug(f"Invoked _change_state: '{state}' with '{kwargs}'")

        if not self._service:
            raise RuntimeError('You need to add a service to this device')

        # throttle requests to service
        last_update = self.last_update
        if last_update < self.UPDATES_THROTTLE:
            remaining = self.UPDATES_THROTTLE - last_update
            message = f'Please wait {remaining} seconds...'
            logging.debug(message)
            return ServiceResponse(False, message)

        # create new parameters
        parameters = self.parameters.create(**kwargs)

        # map the event enum to its string
        service_event = self._state_to_service_event(state)

        # trigger the service
        response = self._service.trigger(
            event_name=service_event,
            parameters=parameters
        )

        # update the state and last-update timestamp
        if response.success:
            self.state = state
            self.update(parameters)

        return response

    @property
    def last_update(self) -> int:
        if self._last_update is None:
            return math.inf

        now = datetime.now()
        duration = now - self._last_update
        return int(duration.total_seconds())

    def update(self, parameters: Parameters) -> None:
        self._parameters = parameters
        self._last_update = datetime.now()


class LightBulb(SmartDevice):

    def __init__(self, name: str, *args, **kwargs):
        super().__init__(name, *args, **kwargs)

    @property
    def default_parameters(self):
        return LightBulbParameters()

    def turn_on(self, **params) -> ServiceResponse:
        return self._change_state(SmartDevice.State.ON, **params)

    def turn_off(self, **params) -> ServiceResponse:
        return self._change_state(SmartDevice.State.OFF, **params)



from enum import Enum
from unittest import TestCase, mock

import pytest
import schema
from symbiotic.services.ifttt import IFTTT


class StatusCodes(Enum):
    # pylint: disable=unsubscriptable-object
    # the above to suppress errors for the instance methods
    # https://github.com/PyCQA/pylint/issues/2063

    OK = (200, 'ok')
    NO_CONTENT = (204, 'no_content')
    BAD_REQUEST = (400, 'bad_request')
    UNAUTHORIZED = (401, 'unauthorized')
    FORBIDDEN = (403, 'forbidden')
    NOT_FOUND = (404, 'not_found')
    INVALID_PARAMETERS = (422, 'invalid_parameters')
    INTERNAL_SERVER_ERROR = (500, 'internal_server_error')
    BAD_GATEWAY = (502, 'bad_gateway')

    @staticmethod
    def by_reason(reason: str) -> 'StatusCodes':
        for item in StatusCodes:
            if reason == item.value[1]:
                return item
        raise Exception(f'Reason not found in status codes: {reason}')

    @staticmethod
    def has_reason(reason: str) -> str:
        values = set(item.value[1] for item in StatusCodes)
        return reason in values

    @property
    def code(self) -> int:
        return self.value[0]

    @property
    def reason(self) -> str:
        return self.value[1]


def ifttt_service_valid_key() -> IFTTT:
    config = {'key': 'clearly_a_valid_key'}
    return IFTTT(config=config)


def ifttt_service_invalid_key() -> IFTTT:
    config = {'key': 'clearly_an_invalid_key'}
    return IFTTT(config=config)


# # This method will be used by the mock to replace requests.post
# # Source: https://stackoverflow.com/questions/15753390/how-can-i-mock-requests-and-the-response
# def mocked_requests_post(*args, **kwargs):
#     from urllib.parse import urlparse

#     class MockResponse:
#         def __init__(self, status_code, *args, **kwargs):
#             self.status_code = status_code
#             self.text = kwargs.get('text')
#             self.json_data = kwargs.get('json_data')

#         def json(self):
#             return self.json_data

#         @property
#         def ok(self):
#             return self.status_code < 400

#     url = args[0]
#     url_segments = urlparse(url).path.split('/')
#     event_name = url_segments[2]
#     key = url_segments[5]

#     # check key
#     if key == 'bad_key':  # key is not valid
#         return MockResponse(StatusCodes.UNAUTHORIZED.code, StatusCodes.UNAUTHORIZED.reason)
#     elif key == 'unauthorized_key':  # permissions not sufficient for action
#         return MockResponse(StatusCodes.FORBIDDEN.code, StatusCodes.FORBIDDEN.reason)
#     elif key == 'valid_key':
#         # key is authorized, check the event_name
#         if event_name == 'non-existent_event':
#             return MockResponse(StatusCodes.NOT_FOUND.code, StatusCodes.NOT_FOUND.reason)
#         elif event_name == 'valid-event_name':
#             return MockResponse(StatusCodes.OK.code, StatusCodes.OK.reason)

#     return MockResponse(StatusCodes.BAD_REQUEST.code, StatusCodes.BAD_REQUEST.reason)


@mock.patch('symbiotic.services.ifttt.requests.post', autospec=True)
class Test_IFTTT_Unit(TestCase):

    def test_trigger_valid_request_no_params(self, mock_post):
        ifttt = IFTTT(config={'key': 'valid_key'})
        response = ifttt.trigger(event_name='name')
        self.assertTrue(response.success)

    def test_trigger_valid_request_with_params1(self, mock_post):
        ifttt = IFTTT(config={'key': 'valid_key'})
        response = ifttt.trigger(event_name='name', parameters={'value1': 42})
        self.assertTrue(response.success)

    def test_trigger_valid_request_with_params2(self, mock_post):
        ifttt = IFTTT(config={'key': 'valid_key'})
        response = ifttt.trigger(event_name='name', parameters={'value2': 42, 'value3': 'some-value'})
        self.assertTrue(response.success)

    def test_trigger_valid_request_with_params3(self, mock_post):
        ifttt = IFTTT(config={'key': 'valid_key'})
        response = ifttt.trigger(event_name='name', parameters={})
        self.assertTrue(response.success)

    def test_trigger_valid_request_with_params4(self, mock_post):
        ifttt = IFTTT(config={'key': 'valid_key'})
        response = ifttt.trigger(event_name='name', parameters={})
        self.assertTrue(response.success)

    def test_trigger_valid_request_with_params5(self, mock_post):
        ifttt = IFTTT(config={'key': 'valid_key'})
        with pytest.raises(schema.SchemaUnexpectedTypeError):
            ifttt.trigger(event_name='name', parameters='test')

    def test_trigger_valid_request_with_params6(self, mock_post):
        ifttt = IFTTT(config={'key': 'valid_key'})
        with pytest.raises(TypeError):
            ifttt.trigger(event_name='name', color='black')

    def test_trigger_valid_request_with_params7(self, mock_post):
        ifttt = IFTTT(config={'key': 'valid_key'})
        with pytest.raises(schema.SchemaWrongKeyError):
            ifttt.trigger(event_name='name', parameters={'value1': 'some-value', 'value4': 55})

    def test_trigger_valid_request_with_params8(self, mock_post):
        ifttt = IFTTT(config={'key': 'valid_key'})
        with pytest.raises(schema.SchemaWrongKeyError):
            ifttt.trigger(event_name='name', parameters={'value1': 'some-value', 'sheep': 'baaah'})



from unittest import TestCase

import pytest
from symbiotic.core import _scheduler
from symbiotic.devices.actions import Action, ActionScheduler


class Test_Action_Unit(TestCase):

    def test_init1(self):
        action = Action()
        self.assertIsNotNone(action)
        self.assertIsNone(action.func)

    def test_init2(self):
        def job():
            return 'fnct-passed-test'

        action = Action(job)
        self.assertIsNotNone(action)
        self.assertIsNotNone(action.func)

    def test_init3(self):
        with pytest.raises(TypeError):
            # no function, but parameters passed
            action = Action(parameters={'some-key': 'some-value'})
            self.assertIsNotNone(action)

    def test_init4(self):
        def job(*args: int):
            return args[0] + 1

        action = Action(job, 9)
        self.assertIsNotNone(action)
        self.assertIsNotNone(action.func)

    def test_init5(self):
        def job(**kwargs):
            value = kwargs.get('value')
            return value + '-passed'

        action = Action(job, 9)
        self.assertIsNotNone(action)
        self.assertIsNotNone(action.func)


class Test_Action_Integration(TestCase):

    def test_call_action_no_args_returns_result(self):
        def job():
            return 'fnct-passed-test'

        action = Action(job)
        value = action()
        self.assertEqual(value, 'fnct-passed-test')

    def test_call_action_with_args_returns_result(self):
        def job(*args):
            return {'key': args[0]}

        action = Action(job, 42)
        value = action()
        self.assertEqual(value['key'], 42)

    def test_call_action_with_kwargs_returns_result(self):
        def job(**kwargs):
            value = kwargs.get('value')
            return value + '-passed'

        action = Action(job, value='test')
        self.assertIsNotNone(action)
        self.assertIsNotNone(action.func)

        value = action()
        self.assertEqual(value, 'test-passed')

    def test_call_action_no_return(self):
        def job():
            print('test')
            pass

        action = Action(job)
        value = action()
        self.assertIsNone(value)

    def test_call_action_should_raise_error(self):
        action = Action()
        with pytest.raises(AttributeError):
            # action w/o job called
            action()


class Test_ActionScheduler_Unit(TestCase):

    def test_init_no_args(self):
        scheduler = ActionScheduler()
        self.assertIsNotNone(scheduler)
        self.assertIsNone(scheduler.func)
        self.assertIsNone(scheduler.name)
        self.assertEqual(len(scheduler.kwargs), 0)
        self.assertEqual(len(scheduler.actions), 0)

    def test_init_with_name_kwargs(self):
        def somefunc(*args):
            pass

        scheduler = ActionScheduler(somefunc, name='scheduler')
        self.assertIsNotNone(scheduler)
        self.assertIsNotNone(scheduler.func)
        self.assertEqual(scheduler.name, 'scheduler')
        self.assertEqual(len(scheduler.kwargs), 0)
        self.assertEqual(len(scheduler.actions), 0)

    def test_init_with_job_kwargs(self):
        def somefunc(**kwargs):
            pass

        scheduler = ActionScheduler(somefunc, someparam='value')
        self.assertIsNotNone(scheduler)
        self.assertIsNotNone(scheduler.func)
        self.assertEqual(len(scheduler.kwargs), 1)
        self.assertEqual(scheduler.kwargs, {'someparam': 'value'})
        self.assertEqual(len(scheduler.actions), 0)

    def test_add_action_with_kwargs1(self):
        def somefunc(**kwargs):
            pass

        scheduler = ActionScheduler(somefunc, someparam=42)
        scheduler.add().every().day

        # check the action has been added
        self.assertEqual(len(scheduler.actions), 1)

        # ensure inner values have been set correctly
        self.assertIsNotNone(scheduler.actions[0])
        action = scheduler.actions[0]
        self.assertEqual(action.func.args, ())
        self.assertEqual(action.func.keywords, {'someparam': 42})
        self.assertIsNotNone(action.job)

        # ensure the job has not been added to the scheduler
        self.assertEqual(len(_scheduler.jobs), 0)

    def test_add_action_with_kwargs2(self):
        def somefunc(**kwargs):
            pass

        scheduler = ActionScheduler(somefunc)
        scheduler.add(someparam='value').every().sunday.at('23:59')

        # check the action has been added
        self.assertEqual(len(scheduler.actions), 1)

        # ensure inner values have been set correctly
        action = scheduler.actions[0]
        self.assertEqual(action.func.args, ())
        self.assertEqual(action.func.keywords, {'someparam': 'value'})
        self.assertIsNotNone(action.job)
        self.assertIsNotNone(scheduler.actions[0])

        # ensure the job has not been added to the scheduler
        self.assertEqual(len(_scheduler.jobs), 0)


class Test_ActionScheduler_Integration(TestCase):
    """ Tests for integration with the `scheduler` lib. """

    @pytest.fixture(autouse=True)
    def run_around_tests(self):
        # Ensure the previous test hasn't left any jobs in the queue
        err = 'The job scheduler queue is not empty'
        assert len(_scheduler.jobs) == 0, err
        yield
        # Clear any jobs added by a test
        _scheduler.clear()
        assert len(_scheduler.jobs) == 0, err

    def test_add_action_with_kwargs1(self):
        def somefunc(**kwargs):
            pass

        scheduler = ActionScheduler(somefunc, someparam='value')
        scheduler.add().every().sunday.at('23:59')
        scheduler._finalize()

        # ensure the job has been added to the scheduler
        self.assertEqual(len(_scheduler.jobs), 1)
        # ensure the action has a reference to the job
        self.assertEqual(_scheduler.jobs[0], scheduler.actions[0].job)

    def test_add_action_with_kwargs2(self):
        def somefunc(**kwargs):
            pass

        scheduler = ActionScheduler(somefunc)
        scheduler.add(someparam='value').every().sunday.at('23:59')
        scheduler._finalize()

        # ensure the job has been added to the scheduler
        self.assertEqual(len(_scheduler.jobs), 1)
        # ensure the action has a reference to the job
        self.assertEqual(_scheduler.jobs[0], scheduler.actions[0].job)

    def test_add_no_action_should_fail1(self):
        scheduler = ActionScheduler()

        with pytest.raises(TypeError):
            scheduler.add().every().sunday.at('23:59')

        # action not added
        self.assertEqual(len(scheduler.actions), 0)
        # ensure the job scheduler queue is empty
        self.assertEqual(len(_scheduler.jobs), 0)

    def test_add_no_action_should_fail2(self):
        scheduler = ActionScheduler(someparam='bla')

        with pytest.raises(TypeError):
            scheduler.add().every().sunday.at('23:59')

        # action not added
        self.assertEqual(len(scheduler.actions), 0)
        # ensure the job scheduler queue is empty
        self.assertEqual(len(_scheduler.jobs), 0)

    def test_clear_jobs(self):
        def somefunc(**kwargs):
            pass

        scheduler = ActionScheduler(somefunc)
        scheduler.add().every().sunday.at('23:59')
        scheduler._finalize()

        self.assertEqual(len(scheduler.actions), 1)
        self.assertEqual(len(_scheduler.jobs), 1)

        scheduler.clear()
        self.assertEqual(len(scheduler.actions), 0)
        self.assertEqual(len(_scheduler.jobs), 0)



import atexit
import functools
import logging
import secrets
from abc import ABC
from contextlib import contextmanager
from typing import Callable, List

from ..core import _event_bus, _scheduler


class Action(object):

    bus = _event_bus
    scheduler = _scheduler

    def __init__(self, func: Callable = None, *args, **kwargs):
        if not func and (args or kwargs):
            raise TypeError('Parameters set, but no function passed.')

        self.name = kwargs.pop('name', secrets.token_hex(16))
        self.func = functools.partial(func, *args, **kwargs) if func else None
        atexit.register(self.unregister)
        super().__init__()

    def __eq__(self, other: 'Action'):
        return self.name == other.name

    def __hash__(self):
        return hash((self.name))

    def __call__(self):
        if self.func is None:
            err = f'The action {self.name} does not have a callable function.'
            raise AttributeError(err)

        return self.func()

    def do(self, func: Callable, *args, **kwargs) -> 'Action':
        self.func = functools.partial(func, *args, **kwargs)
        return self

    def unregister(self) -> None:
        """ Override to unregister the action from its handler. """
        pass


class EventedAction(Action):

    event: str

    def __init__(self, on: str, **kwargs):
        super().__init__(**kwargs)
        self.event = on
        self.bus.add_event(self, self.event)
        logging.debug(f'{self.event} registered on bus')

    def unregister(self) -> None:
        """ Removes the action from the bus. """
        # Dirty hack to make the bus remove events:
        # the bus method does comparison by using instance.__name__ == instance
        # but this fails because class instances of Action don't have __name__
        # Implementing __name__ wouldn't work as we override __eq__, so it
        # would try to compare an instance of Action to a string.
        # This could be solved in a better way, but the lazy one take less time.
        self.__name__ = self
        self.bus.remove_event(self, self.event)
        logging.debug(f'removed {self.event} from bus')


class ScheduledAction(Action):

    def __init__(self, func: Callable = None, *args, **kwargs):
        super().__init__(func, *args, **kwargs)
        self.job = None

    def every(self, interval: int = 1):
        self.job = self.scheduler.every(interval).tag(self.name)
        return self.job

    def register(self):
        if not self.job:
            err = f"Cannot schedule '{self.name}': no schedule specified"
            raise AttributeError(err)

        self.job.do(self)
        logging.debug(f'{self.name} added to the schedule')

    def unregister(self) -> bool:
        self.scheduler.cancel_job(self.job)
        logging.debug(f'{self.name} removed from the schedule')
        return True


class ActionScheduler(object):

    def __init__(self, func: Callable = None, **kwargs):
        self.name = kwargs.pop('name', None)
        self.func = func
        self.kwargs = kwargs
        self.actions = []
        atexit.register(self.clear)
        super().__init__()

    def add(self, func: Callable = None, *args, **kwargs):
        if func is not None:
            # action specified
            action = ScheduledAction(func, *args, **{**self.kwargs, **kwargs})
        else:
            if self.func is None:
                raise TypeError('Cannot schedule an empty action.')
            action = ScheduledAction(
                self.func, *args, **{**self.kwargs, **kwargs})
        self.actions.append(action)
        return action

    def _finalize(self):
        for action in self.actions:
            action.register()

    def clear(self):
        # mutate existing list by adding back only actions if failed to unregister
        self.actions[:] = [act for act in self.actions if not act.unregister()]


class Actionable(ABC):

    actions: List[Action]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.actions = []

    def event(self, name: str, **kwargs) -> Action:
        action = EventedAction(name)
        self.actions.append(action)
        return action

    @contextmanager
    def schedule(self, job: Callable = None, **kwargs) -> ActionScheduler:
        """ Registers the action with the scheduler.

        Using contextmanager allows to yield an instance of ActionScheduler
        and to set the scheduled job's job (i.e. this action) from within
        this method, which gives us more control.

        Example:
            >>> with device.schedule(device.action) as schedule:
            >>>     schedule.add().every().day.at('08.30')
            >>>     schedule.add().every(5).days.at('11.30')
        """
        scheduler = ActionScheduler(job, **kwargs)
        yield scheduler
        scheduler._finalize()



from dataclasses import dataclass
from requests import Response


@dataclass
class ServiceResponse(object):
    success: bool
    message: str

    @staticmethod
    def from_response(response: Response):
        # use raise_for_status with try
        success = response.ok
        return ServiceResponse(success=success, message=response.text)



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="symbiotic",
    version="1.0.0-alpha1",
    author="Stefano Frazzetto",
    author_email="s.frazzetto22+pypi@gmail.com",
    description="Connect your smart devices and sensors to create complex actions and schedules.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/StefanoFrazzetto/symbiotic",
    packages=setuptools.find_packages(),
    python_requires='>=3.6',
    classifiers=[
        'Development Status :: 3 - Alpha',
        "Programming Language :: Python :: 3",
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
        "Topic :: Home Automation",
        "Topic :: Software Development :: Libraries",
        "Topic :: System :: Hardware",
    ],
    keywords=[
        'automation',
        'dependency-injection',
        'event-bus',
        'iot',
        'schedule',
        'security',
        'smart-devices',
    ],
    install_requires=[
        "cached-property",
        "dependency-injector>=4.11",
        "event-bus",
        "gpiozero",
        "pigpio",
        "pydantic",
        "schedule",
        "schema",
        "requests"
    ],
    extras_require={
        'dev': ['check-manifest'],
        'test': [
            "pytest",
            "pytest-mock"
        ],
        'yaml': ["PyYAML>=5.4"]
    },
    project_urls={
        'Bug Reports': 'https://github.com/StefanoFrazzetto/symbiotic/issues',
        'Source': 'https://github.com/StefanoFrazzetto/symbiotic',
    },
)



import logging
from abc import ABC

from gpiozero import MotionSensor as GPIOZeroMotionSensor
from gpiozero.exc import BadPinFactory

from ..core import _event_bus, _scheduler
from ..core.exceptions import ConfigurationError


class MotionSensor(ABC):
    """
    MotionSensor provides an interface for motion sensors.

    When motion is detected, the sensor should call `active`;
    this will emit an event on the bus as `sensor_name:active`.

    Args:
        name (str): the name to associate with the motion sensor.
    """
    name: str
    bus = _event_bus
    scheduler = _scheduler

    def __init__(self, name: str, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.name = name

    @property
    def active(self) -> str:
        return f'{self.name}:active'

    @property
    def inactive(self) -> str:
        return f'{self.name}:inactive'

    def active_hook(self):
        logging.debug(f'{self.name}: movement detected.')
        self.bus.emit(self.active)

    def inactive_hook(self):
        logging.debug(f'{self.name}: movement stopped.')
        self.bus.emit(self.inactive)


class GPIOMotionSensor(MotionSensor):

    def __init__(self, name: str, pin: int, *args, **kwargs):

        try:
            pin_factory = kwargs.pop('pin_factory', None)
            self._sensor = GPIOZeroMotionSensor(pin, pin_factory=pin_factory)
        except BadPinFactory:
            err = f"Cannot instantiate {self.name} without a pin factory!"
            raise ConfigurationError(err)

        super().__init__(name, *args, **kwargs)

        self._sensor.when_motion = self.active_hook
        self._sensor.when_no_motion = self.inactive_hook
